package de.jottyfan.timetrack.modules.done;

import java.io.Serializable;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.Map;

import de.jottyfan.timetrack.db.done.tables.records.TDoneRecord;
import de.jottyfan.timetrack.db.done.tables.records.TJobRecord;
import de.jottyfan.timetrack.db.done.tables.records.TModuleRecord;
import de.jottyfan.timetrack.db.done.tables.records.TProjectRecord;
import de.jottyfan.timetrack.modules.Bean;

/**
 * 
 * @author henkej
 *
 */
public class DoneBean implements Bean, Serializable, Comparable<DoneBean> {
	private static final long serialVersionUID = 1L;

	private Integer pk;
	private Timestamp timeFrom;
	private Timestamp timeUntil;
	private TProjectRecord project;
	private TModuleRecord module;
	private TJobRecord activity;
	private String wp;

	public DoneBean() {
	}

	public DoneBean(TDoneRecord r, Map<Integer, TProjectRecord> projectMap, Map<Integer, TModuleRecord> moduleMap,
			Map<Integer, TJobRecord> jobMap) {
		this.pk = r.getPk();
		this.timeFrom = r.getTimeFrom();
		this.timeUntil = r.getTimeUntil();
		this.project = projectMap.get(r.getFkProject());
		this.module = moduleMap.get(r.getFkModule());
		this.activity = jobMap.get(r.getFkJob());
		this.wp = r.getWp();
	}

	/**
	 * set the day of timeFrom and timeUntil, keeping the times
	 * 
	 * @param day
	 *          the day
	 */
	public void setDay(Date day) {
		if (timeFrom != null) {
			LocalDateTime ldt = timeFrom.toLocalDateTime();
			LocalDate date = day.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
			ldt = ldt.withYear(date.getYear()).withMonth(date.getMonthValue()).withDayOfMonth(date.getDayOfMonth());
			timeFrom = Timestamp.from(ldt.toInstant(OffsetDateTime.now().getOffset()));
		}
		if (timeUntil != null) {
			LocalDateTime ldt = timeUntil.toLocalDateTime();
			LocalDate date = day.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
			ldt = ldt.withYear(date.getYear()).withMonth(date.getMonthValue()).withDayOfMonth(date.getDayOfMonth());
			timeUntil = Timestamp.from(ldt.toInstant(OffsetDateTime.now().getOffset()));
		}
	}

	public String getTimeSummary() {
		StringBuilder buf = new StringBuilder();
		if (timeFrom != null) {
			buf.append(new SimpleDateFormat("HH:mm").format(timeFrom));
		}
		if (timeUntil != null) {
			buf.append(" - ");
			buf.append(new SimpleDateFormat("HH:mm").format(timeUntil));
		}
		return buf.toString();
	}

	@Override
	public int compareTo(DoneBean o) {
		return o == null || timeFrom == null || o.getTimeFrom() == null ? 0 : timeFrom.compareTo(o.getTimeFrom());
	}

	public String getTimeDiff() {
		LocalDateTime earlier = timeFrom != null ? timeFrom.toLocalDateTime() : LocalDateTime.now();
		LocalDateTime later = timeUntil != null ? timeUntil.toLocalDateTime() : LocalDateTime.now();
		Duration diff = Duration.between(earlier, later);
		return String.format("%02d:%02d", diff.toHours(), diff.toMinutes() % 60);
	}

  /**
   * get local date time from s
   * 
   * @param s   the HH:mm formatted values
   * @param ldt the date as basic for that datetime, for today one can use LocalDateTime.now(); in fact this is set if ldt is null internally
   * @return the generated datetime
   */
	public LocalDateTime getLocalDateTimeFromHHmm(String s, LocalDateTime ldt) {
		if (s == null || s.trim().isEmpty()) {
			return null;
		} else if (ldt == null) {
		  ldt = LocalDateTime.now();
		}
		String[] hm = s.split(":");
		Integer hours = 0;
		Integer minutes = 0;
		if (hm.length > 0) {
			hours = Integer.valueOf(hm[0]);
		}
		if (hm.length > 1) {
			minutes = Integer.valueOf(hm[1]);
		}
		ldt = ldt.withHour(hours);
		ldt = ldt.withMinute(minutes);
		ldt = ldt.withSecond(0);
		ldt = ldt.withNano(0);
		return ldt;
	}

  public String getProjectNameWithWP() {
    StringBuilder buf = new StringBuilder();
    buf.append(project == null ? "" : project.getName());
    if (wp != null && !wp.isBlank()) {
      buf.append(" (").append(wp).append(")");
    }
    return buf.toString();
  }

	public String getProjectName() {
		return project == null ? "" : project.getName();
	}

	public String getModuleName() {
		return module == null ? "" : module.getName();
	}

	public String getJobName() {
		return activity == null ? "" : activity.getName();
	}

	public String getTimeFromString() {
		return timeFrom == null ? "" : new SimpleDateFormat("HH:mm").format(timeFrom);
	}

	public void setTimeFromString(String s) {
		LocalDateTime ldt = getLocalDateTimeFromHHmm(s, null); // use setDay instead
		this.timeFrom = ldt == null ? null : Timestamp.valueOf(ldt);
	}

	public String getTimeUntilString() {
		return timeUntil == null ? "" : new SimpleDateFormat("HH:mm").format(timeUntil);
	}

	public void setTimeUntilString(String s) {
		LocalDateTime ldt = getLocalDateTimeFromHHmm(s, null); // use setDay instead
		this.timeUntil = ldt == null ? null : Timestamp.valueOf(ldt);
	}

	public Integer getPk() {
		return this.pk;
	}

	public void setPk(Integer pk) {
		this.pk = pk;
	}

	public Timestamp getTimeFrom() {
		return timeFrom;
	}

	public void setTimeFrom(Timestamp timeFrom) {
		this.timeFrom = timeFrom;
	}

	public Timestamp getTimeUntil() {
		return timeUntil;
	}

	public void setTimeUntil(Timestamp timeUntil) {
		this.timeUntil = timeUntil;
	}

	public TProjectRecord getProject() {
		return project;
	}

	public void setProject(TProjectRecord project) {
		this.project = project;
	}

	public TModuleRecord getModule() {
		return module;
	}

	public void setModule(TModuleRecord module) {
		this.module = module;
	}

	public TJobRecord getActivity() {
		return activity;
	}

	public void setActivity(TJobRecord activity) {
		this.activity = activity;
	}

  /**
   * @return the wp
   */
  public String getWp() {
    return wp;
  }

  /**
   * @param wp the wp to set
   */
  public void setWp(String wp) {
    this.wp = wp;
  }
}
